@@ -7,4 +7,4 @@ class TestManager:
     def __init__(self, app: Sanic) -> None:
         self.test_client = SanicTestClient(app)
         self.asgi_client = SanicASGITestClient(app)
-        app._test_manager = self
+        app._test_manager = self  # type: ignore
